<?php

/**
 * @file
 * Definition of paragraph translation migration form.
 */

namespace Drupal\paragraph_translation_migration\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\node\Entity\Node;
use Drupal\paragraphs\Entity\Paragraph;

class ParagraphTranslationMigrationTriggerForm extends FormBase {


  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'paragraph_translation_migration';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Import'),
    );

    return $form;
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    foreach (\Drupal::getContainer()->get('entity_field.manager')->getFieldMapByFieldType('entity_reference_revisions') as $entity_type => $fields) {
      $entity_type = \Drupal::entityTypeManager()->getDefinition($entity_type);
      //if ($entity_type->id() == 'node') {
        foreach ($fields as $field_name => $field) {
          $field_storage_config = FieldStorageConfig::loadByName($entity_type->id(), $field_name);
          if ($field_storage_config->getSetting('target_type') == 'paragraph') {
            //drupal_set_message($field_storage_config->getLabel());
            //continue;
            foreach ($field['bundles'] as $bundle) {
              //if ($bundle == 'page') {
              $entities = \Drupal::entityTypeManager()->getStorage($entity_type->id())->loadMultiple(\Drupal::entityQuery($entity_type->id())->condition($entity_type->getKey('bundle'), $bundle)->execute());
              foreach ($entities as $entity) {
                //if ($entity->id() == 231) {
                $translated_languages = $entity->getTranslationLanguages(FALSE);
                if (!empty($translated_languages)) {
                  $paragraph_values = $entity->get($field_name)->getValue();
                  $paragraph_entities = [];
                  foreach ($paragraph_values as $item) {
                    $paragraph_entities[$item['target_revision_id']] = \Drupal::entityTypeManager()->getStorage('paragraph')->loadRevision($item['target_revision_id']);
                  }

                  foreach ($translated_languages as $language) {
                    $translated_paragraph_entities = [];
                    foreach ($paragraph_entities as $paragraph_entity) {
                      $translated_paragraph_entity = $paragraph_entity->createDuplicate();
                      if ($translated_paragraph_entity->hasTranslation($language->getId())) {
                        $translated_paragraph_entity->removeTranslation($language->getId());

                        foreach ($paragraph_entity->getFieldDefinitions() as $paragraph_field) {
                          if (!in_array($paragraph_field->getName(), ['langcode', 'uuid', 'id', 'default_langcode'])) {
                            $translated_paragraph_entity->set($paragraph_field->getName(), $paragraph_entity->getTranslation($language->getId())->get($paragraph_field->getName())->getValue());
                          }
                        }

                        $translated_paragraph_entity->save();
                        $translated_paragraph_entities[] = [
                          'target_id' => $translated_paragraph_entity->id(),
                          'target_revision_id' => $translated_paragraph_entity->getRevisionId(),
                        ];
                      }
                    }

                    $translated_entity = $entity->getTranslation($language->getId());
                    $translated_entity->set($field_name, $translated_paragraph_entities);
                    $translated_entity->save();
                  }
                }
                //}
              }
              //}
            }
          }
        }
      //}
    }
  }
}
